#include <AccelStepper.h>

#define LDR_PIN_TL A0 // Top-Left
#define LDR_PIN_BL A1 // Bottom-Left
#define LDR_PIN_TR A2 // Top-Right
#define LDR_PIN_BR A3 // Bottom-Right 

// Initialize steppers
AccelStepper stepperX(AccelStepper::HALF4WIRE, 8, 10, 9, 11);
AccelStepper stepperY(AccelStepper::HALF4WIRE, 4, 6, 5, 7);

// --- Configuration Constants ---
const int TOLERANCE = 30; // Combined tolerance for error signals. 

// Proportional Gain (Kp): Increased for faster response, decrease if it overshoots
const float KP_X = 0.35; 
const float KP_Y = 0.3; 

// LDR Reading
const int LDR_SAMPLES = 5;	
const int LDR_DELAY_MS = 1; 

// Stepper Speed and Acceleration: INCREASED FOR FASTER MOVEMENT
const float MAX_SPEED_X = 4000.0;	  
const float ACCELERATION_X = 50000.0; 
const float MAX_SPEED_Y = 4000.0;	  
const float ACCELERATION_Y = 50000.0; 

// --- Global Variables ---
long offsetX_cal = 0; // Calibration offset for X-axis calculation
long offsetY_cal = 0; // Calibration offset for Y-axis calculation

// Helper function to read LDR value with averaging
int readLDR(int pin)
{
	long sum = 0;
	for (int i = 0; i < LDR_SAMPLES; i++)
	{
		sum += analogRead(pin);
		if (LDR_DELAY_MS > 0)
		{
			delay(LDR_DELAY_MS);
		}
	}
	return sum / LDR_SAMPLES;
}

void setup()
{
	Serial.begin(9600);
	Serial.println("Solar Tracker Initializing (45-deg sensors)...");

	stepperX.setMaxSpeed(MAX_SPEED_X);
	stepperX.setAcceleration(ACCELERATION_X);
	stepperY.setMaxSpeed(MAX_SPEED_Y);
	stepperY.setAcceleration(ACCELERATION_Y);

	Serial.println("Calibrating LDRs... Point sensor at reference light.");
	delay(2000);

	int valTL_cal = readLDR(LDR_PIN_TL);
	int valTR_cal = readLDR(LDR_PIN_TR);
	int valBL_cal = readLDR(LDR_PIN_BL);
	int valBR_cal = readLDR(LDR_PIN_BR);

	// Calculate combined values for calibration
	long sumLeft_cal = valTL_cal + valBL_cal;
	long sumRight_cal = valTR_cal + valBR_cal;
	offsetX_cal = sumLeft_cal - sumRight_cal;

	long sumTop_cal = valTL_cal + valTR_cal;
	long sumBottom_cal = valBL_cal + valBR_cal;
	offsetY_cal = sumTop_cal - sumBottom_cal;

	Serial.print("Calibration offsetX_cal: ");
	Serial.println(offsetX_cal);
	Serial.print("Calibration offsetY_cal: ");
	Serial.println(offsetY_cal);
	Serial.println("Setup Complete. Starting tracking loop.");
}

void loop()
{
	// --- Read LDR Values ---
	int valTL = readLDR(LDR_PIN_TL); // Top-Left
	int valTR = readLDR(LDR_PIN_TR); // Top-Right
	int valBL = readLDR(LDR_PIN_BL); // Bottom-Left
	int valBR = readLDR(LDR_PIN_BR); // Bottom-Right

	// --- Calculate Combined Light for Sides ---
	long sumLightLeft = valTL + valBL;
	long sumLightRight = valTR + valBR;
	long sumLightTop = valTL + valTR;
	long sumLightBottom = valBL + valBR;

	// --- Calculate Differences (Error Signals) ---
	// diffX_raw: Positive if more light on Left diagonal sum, Negative if more on Right
	long diffX_raw = sumLightLeft - sumLightRight;
	long errorX = diffX_raw - offsetX_cal; // Apply calibration

	// diffY_raw: Positive if more light on Top diagonal sum, Negative if more on Bottom
	long diffY_raw = sumLightTop - sumLightBottom;
	long errorY = diffY_raw - offsetY_cal; // Apply calibration

	if (errorX > errorY + 2 * TOLERANCE)
	{
		long stepAdjustmentX = -(long)(errorX * KP_X);
		long newTargetX = stepperX.currentPosition() + stepAdjustmentX;
		stepperX.moveTo(newTargetX);
	}

	// --- X-Axis (Horizontal) Control ---
	if (abs(errorX) > TOLERANCE)
	{
		// Positive errorX means more light on the left side (TL+BL).
		long stepAdjustmentX = -(long)(errorX * KP_X);
		long newTargetX = stepperX.currentPosition() + stepAdjustmentX;
		stepperX.moveTo(newTargetX);
	}

	// --- Y-Axis (Vertical) Control ---
	if (abs(errorY) > TOLERANCE)
	{
		// Positive errorY means more light on the top side (TL+TR).
		long stepAdjustmentY = (long)(errorY * KP_Y);
		long newTargetY = stepperY.currentPosition() + stepAdjustmentY;
		stepperY.moveTo(newTargetY);
	}

	// --- Run Steppers ---
	stepperX.run();
	stepperY.run();
}